/*
TECNISA - TECNOLOGIA EM INFORMTICA

NOME: APURAC_FATMERENMANDRO

DESCRIO:
   PROCEDIMENTO PARA APURAO DO FATURAMENTO MERENDA POR ROMANEIO - MANDATO.

CRIAO:
25/04/2005 - DISNEI H. DE MOURA

@C:\WIN_Evol\SQL\END_CONV\ORACLE\6501_FAISTP_FATURMERENMANDRO.SQL;

--   RAISE_APPLICATION_ERROR(-20501,V_IDTAXAADM, TRUE);
*/

CREATE OR REPLACE PROCEDURE APURAC_FATMERENMANDRO(
   P_CDFILIAPUR  IN FILIAL.CDFILIAL%TYPE,
   P_CDFILIPROC  IN FILIAL.CDFILIAL%TYPE,
   P_CDCLIENTE   IN CLIENTE.CDCLIENTE%TYPE,
   P_DATAINI     IN DATE,
   P_DATAFIM     IN DATE,
   P_NRAPURACFAT IN NOTAFISCAL.NRAPURACFAT%TYPE,
   P_NRLANCTONF  IN NOTAFISCAL.NRLANCTONF%TYPE,
   P_DTINIAPUR   IN DATE  DEFAULT NULL,
   P_DTFINAPUR   IN DATE  DEFAULT NULL,
   P_GERANF      IN NOTAFISCAL.IDBLOQUEADO%TYPE) AS

   V_CDEMPRESA    EMPRESA.CDEMPRESA%TYPE;
   V_CDINSCATAC   INSCRESTAD.CDINSCESTA%TYPE;
   V_CDINSCMUNI   INSCRMUNICIP.CDINSCMUNI%TYPE;
   V_CDSERIE      SERIENF.CDSERIE%TYPE;

   V_CDFILIAPUR   CTRLAPURFAT.CDFILIAPUR%TYPE;
   V_DTINIAPURAC  CTRLAPURFAT.DTINIAPURAC%TYPE;
   V_DTFIMAPURAC  CTRLAPURFAT.DTFIMAPURAC%TYPE;

   V_NRLANCTONF   NOTAFISCAL.NRLANCTONF%TYPE;

   V_NRLANCESTQ   LANCTOESTOQ.NRLANCESTQ%TYPE;

   V_CDTIPOOPER   TIPOOPERAC.CDTIPOOPER%TYPE;

   V_CDTIPOCARD   TIPOCARD.CDTIPOCARD%TYPE;
   V_CDSUBGRPROD  SUBGRUPPROD.CDSUBGRPROD%TYPE;
   V_CDPRODFATUR  PRODUTO.CDPRODUTO%TYPE;
   V_CDARVPROD    PRODUTO.CDARVPROD%TYPE;
   V_NMPRODUTO    PRODUTO.NMPRODUTO%TYPE;


   V_VRFATOCONV   NUMBER;
   V_VRFATUMEREND NUMBER;
   V_QTOCORRERFAT NUMBER;
   V_VRTAXAADM    NUMBER;
   V_VRLIQUFATU   NUMBER;
   V_QTITEMPROD   NUMBER;
   V_QTITEMROMA   NUMBER;

   V_PRXMRDP      NUMBER;

   V_GERANF       VARCHAR2(01);
   V_VERIGRUPDES  VARCHAR2(02);
   V_LANCAMENTO   VARCHAR2(20);
   V_CDINSCESTA   VARCHAR2(16);
   V_CDGRUPOFISC  VARCHAR2(03);

   V_INSCMUNI     INSCRMUNICIP.CDINSCMUNI%TYPE;

   V_MAXITENSNF   NUMBER;
   V_NRITEM       NUMBER;

   /*CURSOR AUXILIAR PARA DELETAR AS NOTAS FISCAIS*/
   CURSOR C_DELETANF IS SELECT CDFILIAL, NRLANCTONF
                        FROM NOTAFISCAL
                        WHERE CDFILIAL    = P_CDFILIPROC AND
                              NRAPURACFAT = P_NRAPURACFAT;

   CURSOR C_DESPADMFATUR IS  SELECT L.CDFILIAL, L.CDDESPADMFAT, L.DTLANCDESFAT,  L.VRLANCDESFAT,
                                    D.CDGRUPOFATUR, D.CDPRODUTO, D.DSDESPADMFAT, NVL(L.VRTXLANCDESP,0) VRTAXADMFATU
                             FROM LANCDESPFATU L,  DESPADMFATUR D
                             WHERE (L.CDFILIAL     = P_CDFILIAPUR)                        AND
                                   (D.CDDESPADMFAT = L.CDDESPADMFAT)                      AND
                                   (L.DTLANCDESFAT BETWEEN P_DATAINI AND P_DATAFIM)

                             GROUP BY D.CDGRUPOFATUR, L.CDFILIAL, L.CDDESPADMFAT, L.DTLANCDESFAT, L.VRLANCDESFAT,
                                      D.CDPRODUTO, D.DSDESPADMFAT, VRTXLANCDESP;


   CURSOR C_ITEMROMA IS
      SELECT I.CDFILIFATUR, T.CDFILCENTPME, R.CDFILIAL, T.CDTIPOCARD, T.NMTIPOCARD,
             PT.CDARVPROD, PT.CDPRODUTO, PT.NMPRODUTO,
             (I.VRUNITROMA)  AS VRBRUTITEM,
             NVL(SUM(I.QTITEMROMA),1) AS QTITEMPROD
      FROM ROMANEIO R,   ITEMROMANEIO I, PRODUTO PT, TIPOCARD T, FILITIPOCARD F
      WHERE (R.CDFILIFATUR      = P_CDFILIPROC)          AND
            (R.NRAPURACFAT      = P_NRAPURACFAT)         AND
            (I.CDFILIFATUR      = P_CDFILIPROC)          AND
            (I.NRAPURACFAT      = P_NRAPURACFAT)         AND
            (R.CDFILIAL         = I.CDFILIAL)            AND
            (R.NRROMANEIO       = I.NRROMANEIO)          AND
            (I.CDPRODUTO        = PT.CDPRODUTO)          AND
            (F.CDFILCENTPME     = P_CDFILIAPUR)          AND
            (F.CDFILIAL         = R.CDFILIRECERO)        AND
            (F.IDFATUPROD       = 'S')                   AND
            (F.CDFILCENTPME     = T.CDFILCENTPME)        AND
            (F.CDTIPOCARD       = T.CDTIPOCARD)
      GROUP BY  I.CDFILIFATUR, T.CDFILCENTPME, R.CDFILIAL, T.CDTIPOCARD, T.NMTIPOCARD,
               PT.CDARVPROD, PT.CDPRODUTO, PT.NMPRODUTO, I.VRUNITROMA
      ORDER BY T.CDTIPOCARD;

BEGIN

   V_GERANF       := 'S';
   V_NRITEM       := 0;
   V_VRLIQUFATU   := 0;
   V_QTOCORRERFAT := 0;
   V_QTITEMPROD   := 0;
   V_QTITEMROMA   := 0;
   V_VERIGRUPDES  := '';

   /* FILIAL QUE EST SENDO APURADA (P_FILIAL = FILIPROC)*/
   SELECT MAX(CDFILIAPUR), MAX(DTINIAPURAC)
   INTO V_CDFILIAPUR, V_DTINIAPURAC
   FROM CTRLAPURFAT
   WHERE CDFILIAL    = P_CDFILIPROC   AND
         NRAPURACFAT = P_NRAPURACFAT;



   SELECT MAX(DTINIAPURAC), MAX(DTFIMAPURAC)
   INTO  V_DTINIAPURAC, V_DTFIMAPURAC
   FROM CTRLAPURFAT
   WHERE CDFILIAL    = V_CDFILIAPUR   AND
         DTINIAPURAC < V_DTINIAPURAC;

   SELECT MAX(SUBSTR(PF.DSPARAMFATFI,70,2))  /*TIPO DE OPERACAO  */
   INTO V_CDTIPOOPER
   FROM PARAMFILIAL PF
   WHERE PF.CDFILIAL = P_CDFILIAPUR;

   IF (V_CDTIPOOPER IS NULL) OR (V_CDTIPOOPER = '  ') THEN
      raise_application_error(-20501, 'Tipo de Operao para Apurao no cadastrado.', True);
   END IF;

   SELECT MAX(CDSERIE), MAX(CDINSCESTA), MAX(CDINSCMUNI), MAX(CDEMPRESA)
   INTO V_CDSERIE, V_CDINSCATAC, V_CDINSCMUNI, V_CDEMPRESA
   FROM FILIAL F
   WHERE F.CDFILIAL  = V_CDFILIAPUR;

   /*VERIFICA A QTE MAXIMA DE PRODUTO POR SERIE*/
   SELECT NVL(MAX(NRMAXITNF),10)
   INTO V_MAXITENSNF
   FROM SERIENF
   WHERE CDEMPRESA  = V_CDEMPRESA   AND
         CDINSCESTA = V_CDINSCATAC  AND
         CDINSCMUNI = V_CDINSCMUNI  AND
         CDSERIE    = V_CDSERIE;


   V_NRLANCTONF := P_NRLANCTONF;
   V_LANCAMENTO := P_NRLANCTONF;

   IF P_GERANF = 'S' THEN
	   FOR C_NOTAFISCAL IN C_DELETANF LOOP
   	   DELETA_NF(C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF);
	   END LOOP;
   END IF;

   FOR C_GERANFVENDA IN C_ITEMROMA LOOP
      V_QTITEMPROD   := 0;


         IF V_GERANF = 'S' THEN

            /*INCLUSO DO CORPO DA NOTA*/
            GERA_NOTAFISCAL_CORPO(P_CDFILIPROC, V_CDFILIAPUR, V_CDEMPRESA, V_CDTIPOOPER,
                                  V_CDSERIE, P_CDCLIENTE, P_DATAFIM, 'FAI',
                                  V_LANCAMENTO, V_NRLANCESTQ, 'N', P_NRAPURACFAT,
                                  V_LANCAMENTO, NULL, NULL, NULL, NULL, NULL, NULL,
                                  NULL, NULL, NULL,   NULL, NULL, NULL, NULL,
                                  P_DATAFIM, NULL);

            V_GERANF := 'N';

            V_CDTIPOCARD  := C_GERANFVENDA.CDTIPOCARD;

         END IF;


         IF (V_MAXITENSNF > V_NRITEM) AND (V_CDTIPOCARD = C_GERANFVENDA.CDTIPOCARD)  THEN

            FOR C_GERAOCORR IN (SELECT CDFILIROMA, NRROMANEIO,CDPRODUTO, QTOCORRERFAT
                                FROM OCORRERFATUR
                                WHERE CDFILIROMA  = C_GERANFVENDA.CDFILIAL   AND
                                      CDPRODUTO   = C_GERANFVENDA.CDPRODUTO) LOOP
               SELECT MAX(QTITEMROMA)
               INTO V_QTITEMROMA
               FROM ITEMROMANEIO
               WHERE CDFILIAL    = C_GERAOCORR.CDFILIROMA   AND
                     NRROMANEIO  = C_GERAOCORR.NRROMANEIO AND
                     CDPRODUTO   = C_GERAOCORR.CDPRODUTO;

               IF C_GERAOCORR.QTOCORRERFAT > V_QTITEMROMA THEN
                  V_QTITEMPROD := V_QTITEMPROD +C_GERAOCORR.QTOCORRERFAT;
               ELSIF C_GERAOCORR.QTOCORRERFAT < V_QTITEMROMA THEN
                  V_QTITEMPROD := V_QTITEMPROD + (V_QTITEMROMA - C_GERAOCORR.QTOCORRERFAT);
               ELSE
                  V_QTITEMPROD := V_QTITEMPROD +V_QTITEMROMA;
               END IF;
            END LOOP;

            IF V_QTITEMPROD = 0 THEN
               V_QTITEMPROD := C_GERANFVENDA.QTITEMPROD;
            END IF;

            SELECT MAX(VRFATUMEREND), MAX(CDPRODFATUR), MAX(P.VRFATOCONV)
            INTO V_VRFATUMEREND, V_CDPRODFATUR, V_VRFATOCONV
            FROM ITEMVENDMERE I, PRODUTO P
            WHERE C_GERANFVENDA.CDFILCENTPME = I.CDFILCENTPME AND
                  C_GERANFVENDA.CDPRODUTO    = I.CDPRODUTO    AND
                  C_GERANFVENDA.CDPRODUTO    = P.CDPRODUTO;

            IF V_VRFATOCONV > 0 THEN
               V_QTITEMPROD := V_QTITEMPROD * V_VRFATOCONV;
            END IF;

            SELECT MAX(CDARVPROD), MAX(NMPRODUTO)
            INTO V_CDARVPROD, V_NMPRODUTO
            FROM PRODUTO
            WHERE CDPRODUTO = V_CDPRODFATUR;


            GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_LANCAMENTO, NVL(C_GERANFVENDA.CDPRODUTO,V_CDPRODFATUR),
                                    NVL(V_CDARVPROD,C_GERANFVENDA.CDARVPROD),NVL(V_NMPRODUTO,C_GERANFVENDA.NMPRODUTO),V_QTITEMPROD, NVL(V_VRFATUMEREND,C_GERANFVENDA.VRBRUTITEM), 0,
                                    NULL, NULL, NULL, NULL,
                                    NULL, NULL);

            GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_LANCAMENTO);

            V_NRITEM := V_NRITEM + 1;
         ELSE

            V_CDTIPOCARD  := C_GERANFVENDA.CDTIPOCARD;

            NOVO_CODIGO('LANCNF/FILIAL' || P_CDFILIAPUR,V_LANCAMENTO);
            V_LANCAMENTO := SUBSTR(V_LANCAMENTO,11,10);


            GERA_NOTAFISCAL_CORPO(P_CDFILIPROC, V_CDFILIAPUR, V_CDEMPRESA, V_CDTIPOOPER,
                                  V_CDSERIE, P_CDCLIENTE, P_DATAFIM, 'FAI',
                                  V_LANCAMENTO, V_NRLANCESTQ, 'N', P_NRAPURACFAT,
                                  V_LANCAMENTO,NULL, NULL, NULL, NULL, NULL, NULL,
                                  NULL, NULL,  NULL,  NULL, NULL, NULL, NULL,
                                  P_DATAFIM, NULL);


            FOR C_GERAOCORR IN (SELECT CDFILIROMA, NRROMANEIO,CDPRODUTO, QTOCORRERFAT
                                FROM OCORRERFATUR
                                WHERE CDFILIROMA  = C_GERANFVENDA.CDFILIAL   AND
                                      CDPRODUTO   = C_GERANFVENDA.CDPRODUTO) LOOP
               SELECT MAX(QTITEMROMA)
               INTO V_QTITEMROMA
               FROM ITEMROMANEIO
               WHERE CDFILIAL    = C_GERAOCORR.CDFILIROMA   AND
                     NRROMANEIO  = C_GERAOCORR.NRROMANEIO AND
                     CDPRODUTO   = C_GERAOCORR.CDPRODUTO;

               IF C_GERAOCORR.QTOCORRERFAT > V_QTITEMROMA THEN
                  V_QTITEMPROD := V_QTITEMPROD +C_GERAOCORR.QTOCORRERFAT;
               ELSIF C_GERAOCORR.QTOCORRERFAT < V_QTITEMROMA THEN
                  V_QTITEMPROD := V_QTITEMPROD + (V_QTITEMROMA - C_GERAOCORR.QTOCORRERFAT);
               ELSE
                  V_QTITEMPROD := V_QTITEMPROD + V_QTITEMROMA;
               END IF;
            END LOOP;

            IF V_QTITEMPROD = 0 THEN
               V_QTITEMPROD := C_GERANFVENDA.QTITEMPROD;
            END IF;

            SELECT MAX(VRFATUMEREND), MAX(CDPRODFATUR), MAX(P.VRFATOCONV)
            INTO V_VRFATUMEREND, V_CDPRODFATUR, V_VRFATOCONV
            FROM ITEMVENDMERE I, PRODUTO P
            WHERE C_GERANFVENDA.CDFILCENTPME = I.CDFILCENTPME AND
                  C_GERANFVENDA.CDPRODUTO    = I.CDPRODUTO    AND
                  C_GERANFVENDA.CDPRODUTO    = P.CDPRODUTO;

            IF V_VRFATOCONV > 0 THEN
               V_QTITEMPROD := V_QTITEMPROD * V_VRFATOCONV;
            END IF;

            SELECT MAX(CDARVPROD), MAX(NMPRODUTO)
            INTO V_CDARVPROD, V_NMPRODUTO
            FROM PRODUTO
            WHERE CDPRODUTO = V_CDPRODFATUR;

            GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_LANCAMENTO, NVL(C_GERANFVENDA.CDPRODUTO,V_CDPRODFATUR),
                                    NVL(V_CDARVPROD,C_GERANFVENDA.CDARVPROD),NVL(V_NMPRODUTO,C_GERANFVENDA.NMPRODUTO),V_QTITEMPROD, NVL(V_VRFATUMEREND,C_GERANFVENDA.VRBRUTITEM), 0,
                                    NULL, NULL, NULL, NULL,
                                    NULL, NULL);

            GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_LANCAMENTO);

            V_NRITEM := 1;
         END IF;

   END LOOP;

   /*INCLUSO DO ITENS DA DESPESA ADMINISTRATIVAS LANADAS NO FAM*/
   V_VERIGRUPDES := 'X';

   FOR C_DESPESA IN C_DESPADMFATUR LOOP

      IF (LTRIM(V_CDFILIAPUR) IS NULL) THEN

         INSERT INTO CTRLAPURFAT
            (CDFILIAL, NRAPURACFAT,  CDFILIAPUR, CDCLIENTE, DTINIAPURAC, DTFIMAPURAC, CDTIPOFATUCT)
             VALUES
            (P_CDFILIPROC, P_NRAPURACFAT, P_CDFILIAPUR, P_CDCLIENTE, P_DATAINI, P_DATAFIM, '031');

      END IF;

      IF V_VERIGRUPDES <> C_DESPESA.CDGRUPOFATUR THEN

         IF (V_VERIGRUPDES <> 'X') THEN
           /*EFETUA O FECHAMENTO DA NOTA, GERACAO DE IMPOSTOS*/
           GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_NRLANCTONF);
         END IF;

         V_NRLANCTONF := LPAD(TO_CHAR((TO_NUMBER(V_NRLANCTONF) + 1)),10,'0');

         /*INCLUSO DO CORPO DA NOTA */
         GERA_NOTAFISCAL_CORPO(P_CDFILIPROC, P_CDFILIAPUR, V_CDEMPRESA, V_CDTIPOOPER,
                                  V_CDSERIE, P_CDCLIENTE, P_DATAFIM, 'FAI',
                                  V_NRLANCTONF, V_NRLANCESTQ, 'N', P_NRAPURACFAT,
                                  V_NRLANCTONF,NULL, NULL, NULL, NULL, NULL, NULL,
                                  NULL, NULL, NULL, NULL,  NULL, NULL, NULL,
                                  P_DATAFIM, NULL);

      END IF;

      V_VERIGRUPDES := C_DESPESA.CDGRUPOFATUR;

      IF C_DESPESA.VRTAXADMFATU <> 0  THEN
         V_VRTAXAADM := (C_DESPESA.VRLANCDESFAT * (C_DESPESA.VRTAXADMFATU / 100));
      END IF;

      V_VRLIQUFATU  := (C_DESPESA.VRLANCDESFAT + NVL(V_VRTAXAADM,0));


         /*INCLUSO DOS ITENS DA NOTA FISCAL*/
      IF V_VRLIQUFATU > 0 THEN

         GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_NRLANCTONF, C_DESPESA.CDPRODUTO,
                                 C_DESPESA.CDDESPADMFAT, C_DESPESA.DSDESPADMFAT, 1, V_VRLIQUFATU, 0,
                                 NULL, NULL, NULL, NULL,
                                 NULL, NULL);

         UPDATE LANCDESPFATU
         SET CDFILILANC = P_CDFILIPROC,
             NRLANCTONF = V_NRLANCTONF
         WHERE (CDFILIAL = P_CDFILIAPUR)                        AND
               (CDDESPADMFAT = C_DESPESA.CDDESPADMFAT)          AND
               (DTLANCDESFAT BETWEEN P_DATAINI AND P_DATAFIM);

      END IF;

   END LOOP;

   IF V_VERIGRUPDES  <> 'X'  THEN
       GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_NRLANCTONF);
   END IF;

END;
/

-- VERSAO --
BEGIN ATUVEROBJ('APURAC_FATMERENMAND','5.05.004'); END;
/

